// EEPROM Programmer
// Chris Ward, 1998-2000

// hardware.cpp
//  - Parallel port access
//  - EEPROM programming/reading code

// There's still some Windows-specific stuff in here
#include <windows.h>
#include <commdlg.h>    // common dialog functionality
#include <dlgs.h>       // common dialog template defines
#include <cderr.h>      // common dialog error codes
#include <commctrl.h>	// common controls
#include "resource.h"
#include <math.h>

#include "global_win32.h"
#include "hardware.h"
#include "misc.h"

extern HINSTANCE hInst;
extern HWND MainhWnd;
extern unsigned long devsize, baseaddr;	// device size (bytes), base address
extern dev_code devtype;				// device type (see #defines in header file)
extern LPFNDLLFUNC1 Output; // Function pointer 
extern LPFNDLLFUNC2 Input; // Function pointer 

static unsigned short lptbase, d_reg, c_reg, s_reg;	// parallel port registers


void setuplpt(unsigned short port)
{
	// define i/o addresses for the three lpt registers
	lptbase = port;
	d_reg = lptbase;
	c_reg = lptbase+2;
	s_reg = lptbase+1;

	// set initial state of output lines
	Output(d_reg, 1 + 32);
	Output(c_reg, 0);

	// reset the programmer's shift registers
	Output(d_reg, 0 + 32);		// MR = low
	Sleep(1);
	Output(d_reg, 1 + 32);		// MR = high
}


void ProgramChip(unsigned char *buf)
{
	int multiple, count, progress;
	unsigned long addr;
	HWND hwndPB;

	SetWindowText(GetDlgItem(MainhWnd, STC_STATUS), "Programming");
	hwndPB = GetDlgItem(MainhWnd, PB_PROGRAM);
	if (IsDlgButtonChecked(MainhWnd, CHK_PAGEMODE) == BST_CHECKED)
		multiple = 32;
	else
		multiple = 1;

	Output(c_reg, ENABLE);				// enable EEPROM
	Sleep(10);							// 10ms warm-up time

	for (addr = 0; addr < devsize; addr += multiple)
	{
		for (count = 0; count < multiple; count++)
			writebyte(addr + count, buf[addr + count]);

		while (testbit(Input(s_reg), 3));	// wait for the R/B pin to go low
		while (!testbit(Input(s_reg), 3));	// then wait for it to go high

		progress = ((float)addr / devsize) * 100;
		SendMessage(hwndPB, PBM_SETPOS, (WPARAM) progress, 0);	// update progress bar
	}

	Output(c_reg, 0);			// disable EEPROM

	SetWindowText(GetDlgItem(MainhWnd, STC_STATUS), "Done");
}


void ReadChip(unsigned char *buf)
{
	unsigned long addr;
	int progress;
	HWND hwndPB;

	SetWindowText(GetDlgItem(MainhWnd, STC_STATUS), "Reading");
	hwndPB = GetDlgItem(MainhWnd, PB_PROGRAM);

	Output(c_reg, ENABLE);				// enable EEPROM
	Sleep(10);							// 10ms warm-up time
	Output(d_reg, 1 + 32);				// '164 MR high; '165 shift/load high
	
	for (addr = 0; addr < devsize; addr++)
	{
		buf[addr] = readbyte(addr);		// read a byte from the chip

		progress = ((float)addr / devsize) * 100;
		SendMessage(hwndPB, PBM_SETPOS, (WPARAM) progress, 0);	// update progress bar
	}

	Output(c_reg, 0);			// disable EEPROM

	SetWindowText(GetDlgItem(MainhWnd, STC_STATUS), "Done");
}


void VerifyChip(unsigned char *buf)
{
	unsigned long addr;
	int progress;
	HWND hwndPB;

	SetWindowText(GetDlgItem(MainhWnd, STC_STATUS), "Verifying");
	hwndPB = GetDlgItem(MainhWnd, PB_PROGRAM);

	Output(c_reg, ENABLE);				// enable EEPROM
	Sleep(10);							// 10ms warm-up time
	Output(d_reg, 1 + 32);				// '164 MR high; '165 shift/load high
	
	for (addr = 0; addr < devsize; addr++)
	{
		progress = ((float)addr / devsize) * 100;
		SendMessage(hwndPB, PBM_SETPOS, (WPARAM) progress, 0);	// update progress bar
		if (readbyte(addr) != buf[addr])
		{
			MessageBox(MainhWnd, "Verify error", "Error", MB_OK | MB_ICONERROR);
			return;
		}
	}

	Output(c_reg, 0);			// disable EEPROM
}


void writebyte(unsigned long addr, unsigned char byte)
{
	bool a, b, c;
	int d, bit;

	for (bit = 7; bit >= 0; bit--)
	{
		// set the shift registers' serial inputs
		a = testbit(addr, bit);		// A0-A7
		b = testbit(addr, bit+8);	// A8-A12
		c = testbit(byte, bit);		// D0-D7
		d = a*4 + b*8 + c*16;
		Output(d_reg, d + 1);		// always set bit 0 (MR)

		// send a clock pulse (data port bit 1 -> CP)
		Output(d_reg, d + 1 + 2);
		Output(d_reg, d + 1);
	}

	// send a write pulse
	Output(c_reg, ENABLE + WRITE);
	Output(c_reg, ENABLE);
}


unsigned char readbyte(unsigned long addr)
{
	bool a, b;
	int d, bit;
	unsigned char byte = 0;

	for (bit = 7; bit >= 0; bit--)
	{
		// set the shift registers' serial inputs
		a = testbit(addr, bit);		// A0-A7
		b = testbit(addr, bit+8);	// A8-A14
		d = a*4 + b*8;
		Output(d_reg, d + 1 + 32);	// always set bit 0 (164 MR), bit 5 (165 shift/load)

		// send a clock pulse (data port bit 1 -> CP)
		Output(d_reg, d + 1 + 2 + 32);
		Output(d_reg, d + 1 + 32);
	}

	Output(c_reg, ENABLE + OUTPUT);		// set EEPROM for output

	// load the '165 from the data lines
	Output(d_reg, 1);
	Output(d_reg, 1 + 32);

	Output(c_reg, ENABLE);				// disable EEPROM output

	for (bit = 7; bit >= 0; bit--)
	{
		if (testbit(Input(s_reg), 4))	// read bit from '165
			byte += pow(2, bit);
		Output(d_reg, 1 + 32 + 64);		// shift
		Output(d_reg, 1 + 32);
	}

	return byte;
}
